    processor 6502

    LIST OFF
    include vcs.h
    include macro.h
    include music.h
    LIST ON

; Compile switches

NTSC                    = 0
PAL                     = 1 
COMPILE_VERSION         = NTSC

; Equates

BALLOONFRAMES           = 14
KERNEL_HEIGHT           = $4E
BALLOON_HEIGHT          = 11
PF_LINES                = 26
VERBOTTOMLIMIT          = BALLOON_HEIGHT*2
VERTOPLIMIT             = KERNEL_HEIGHT*2
HORRIGHTLIMIT           = $8E
MAXTIMER                = $44
MAXLIVES                = 4
;MAXLEVELS              = 17
MAXLEVELS               = 4
ENEMYDATASIZE           = 5

; Variables

    SEG.U vars
    ORG $80

frameCounter        ds 1
balloonAnim         ds 1
balloonPtr          ds 2
balloonPtr2         ds 2
balloonmovPtr       ds 2
balloonmovPtr2      ds 2
balloonOffset       ds 1
shiftval            ds 1
charPtr             ds 12

; level data
PFPtr               ds 8
verPosBalloon       ds 1
horPosBalloon       ds 1
enemyType           ds 1
DATASIZE = *-PFPtr


lines               ds 1
bcdScore            ds 3
tempVar1            ds 1
tempVar2            ds 1
levelTimer          ds 1
lives               ds 1
switchesRead        ds 1
level               ds 1

; enemy data
verPosEnemy         ds 1
horPosEnemy         ds 1
enemyHeight         ds 1
enemyPtr            ds 2    ; enemy data end
ENEMYDATASIZE = *-verPosEnemy

enemyOffset         ds 1
enemyAnim           ds 1

    SEG     Bank0
    ORG $F000

;#######################
;# MAIN SCREEN DISPLAY #
;#######################

SkipBalloon
                SLEEP 2
                LDA #$FF
                STA tempVar2
                LDA #$00
                BEQ BalloonContinue
SkipBalloon2
                SLEEP 7
                LDA #$00        ; TXA?
                BEQ BalloonContinue2

; Start display kernel
Maindisplay
        LDY shiftval
        LDX shifttab,Y
ShiftMore
        STA WSYNC
        STA VBLANK          ; Stop vertical blank
        DEX
        BPL ShiftMore

        JSR DrawScore

        LDX #$FF
        STX PF1
        STX PF2
        INX
        STA HMCLR
        LDY #KERNEL_HEIGHT
        STA WSYNC
        STX PF1
        STX PF2

NextLine    
        LDA #BALLOON_HEIGHT-1
        DCP balloonOffset
        BCC SkipBalloon
        LDA (balloonmovPtr),Y
        STA HMP0
        LDA (balloonPtr),Y
BalloonContinue
        STA WSYNC
        STA HMOVE
        STY tempVar1
        STA GRP0
        STX GRP1
        LDA divideby3,Y
        TAY
        LDA (PFPtr),Y
        STA PF1
        LDA (PFPtr+2),Y
        STA.w PF2
        LAX (PFPtr+6),Y
        LDA (PFPtr+4),Y
        STA PF2
        STX PF1
        LDY tempVar1
        LDA tempVar2
        BMI SkipBalloon2
        LDA (balloonmovPtr2),Y
        STA HMP0
        LDA (balloonPtr2),Y
BalloonContinue2
        STA HMOVE
        LDX #$00
        STA GRP0
        STX PF1
        STX PF2
        STX tempVar2

        LDA enemyHeight
        DCP enemyOffset
        BCS .DoEnemy
        TXA
        .byte $2c
.DoEnemy
        LAX (enemyPtr),Y

        SLEEP 17

        DEY
        BNE NextLine      ;get ready for next scanline

        STA WSYNC
        STY GRP0
        STY REFP1

        LDX #$FF
        STX PF1
        STX PF2

        LDA levelTimer
        AND #$FC
        CLC
        ADC #$13
        STA tempVar1

; State display
        LDX lives
        LDA LIVESNUSIZ,X
        STA NUSIZ0
        LDA LIVESOFFSET,X
        SEC
        LDY #$00
        STA WSYNC                ;
        STY CTRLPF
        STY PF1
        STY PF2
.waitzone1
        SBC #$0F                 ;
        BCS .waitzone1            ; RESP loop
        EOR #$07                 ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        STA HMP0
        STA RESP0

        LDA tempVar1
        STA WSYNC                ;
        SEC
.waitzone2
        SBC #$0F                 ;
        BCS .waitzone2
        EOR #$07                 ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        STA HMP1
        STA RESP1
        STA WSYNC
        STA HMOVE

        LDA levelTimer
        LSR
        LSR
        LSR
        LSR
        TAX
        LDY TIMERCOLOR,X
        STY COLUPF
        STY COLUP1

        LDA levelTimer
        LSR
        LSR
        TAX
        LDA levelTimer
        AND #$03
        TAY
        LDA TIMERSHAPE,Y
        STA tempVar1

        LDY #$07
NextStateLine
        STA WSYNC
        LDA #$00
        STA GRP0
        LDA tempVar1
        STA GRP1
        LDA TIMERPF1,X
        STA PF1
        LDA TIMERPF2,X
        STA PF2
        LDA LIVESSHAPE,Y
        STA GRP0
        LDA #$00
        STA PF1
        STA HMP0
        STA HMP1
        SLEEP 2
        STA PF2
        DEY
        BPL NextStateLine
        INY
        STY WSYNC
        STY GRP1
        STY NUSIZ0
        STY GRP0
        STY GRP1
        INY
        STY CTRLPF

        LDA shiftval
        EOR #$08
        TAY
        LDX shifttab,Y
        INX
        INX
        INX
ShiftMore2
        STA WSYNC
        DEX
        BPL ShiftMore2

        LDX  #$1E
        JMP Overscan

;########################
;# CARTRIDGE MAIN ENTRY #
;########################

Start
    CLEAN_START
    JSR InitGame
    STA CTRLPF
    SET_POINTER balloonPtr, BALLOON18
    SET_POINTER balloonPtr2, BALLOON18

; Main Program Loop

NEWSCREEN
    VERTICAL_SYNC

; Set VBLANK timer
    IF COMPILE_VERSION = PAL
        LDA #73
    ELSE
        LDA #43
    ENDIF
    STA TIM64T

    LDA verPosBalloon
    LSR
    STA tempVar1

    LDX balloonAnim
    LDA #KERNEL_HEIGHT
    SEC
    SBC tempVar1
    ADC #BALLOON_HEIGHT-1
    STA balloonOffset
    CLC
    ADC balloonAnimTab,X
    STA balloonPtr
    CLC
    ADC BALLOON_HEIGHT
    STA balloonPtr2

    LDA balloonOffset
    CLC
    ADC balloonMoveTab,X
    STA balloonmovPtr
    CLC
    ADC BALLOON_HEIGHT
    STA balloonmovPtr2
    LDA balloonMoveTabHigh,X
    STA balloonmovPtr+1
    STA balloonmovPtr2+1

    LDA verPosBalloon
    LSR
    BCC PosOk
    LDA balloonPtr
    LDX balloonPtr2
    STA balloonPtr2
    STX balloonPtr
    LDA balloonmovPtr
    LDX balloonmovPtr2
    STA balloonmovPtr2
    STX balloonmovPtr
    LDA balloonmovPtr+1
    LDX balloonmovPtr2+1
    STA balloonmovPtr2+1
    STX balloonmovPtr+1
    DEC balloonPtr2
    DEC balloonmovPtr2
PosOk    

    LDA enemyAnim
    LSR
    LSR
    LSR
    LSR
    TAX
    LDA #KERNEL_HEIGHT
    SEC
    SBC verPosEnemy
    ADC enemyHeight
    STA enemyOffset
    CLC
    ADC faceAnimTab,X
    STA enemyPtr

; Prepare Scoredisplay
    LDA #%01011
    STA NUSIZ0
    STA NUSIZ1
    STA VDELP0
    STA VDELP1
    STA WSYNC
    LDX #7
.Wait
    DEX
    BNE .Wait
    STA.W RESP0
    STA RESP1
    LDA #$14
    STA HMP1
    LSR
    STA HMP0
    STA WSYNC
    STA HMOVE
    LDX #11
NextDigit
    TXA
    LSR
    LSR
    TAY
    LDA bcdScore,Y      ; load low/high values of states
    STA tempVar1
    AND #$0F            ; select lower nibble
    TAY                 ; X-> shape of value
    LDA #>zero          ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    STA charPtr-2,X       ; Store sprite pointer
    DEX
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    DEX 
    LDA tempVar1         ; load low/high values of states
    LSR                 ; select higher nibble... 
    LSR                 ; ...of status value (bcd!)
    LSR                 ; (numbers 0-9)
    LSR                 ;
    TAY                 ; X-> shape of value
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    BPL NextDigit
    LDA #$06
    STA lines
    LDA #$0F
    STA COLUP0 
    STA COLUP1
    LDA #$AA
    STA COLUPF         ; Sprite colour


VBLANKLOOP
    LDA INTIM
    BNE VBLANKLOOP  ; WAIT FOR VBLANK TIMER
    JMP Maindisplay

Overscan

; Set Overscan timer
    IF COMPILE_VERSION = PAL
        LDA #65
    ELSE
        LDA #36
    ENDIF
    STA TIM64T

; Collission Detection
    LDA CXP0FB
    ASL
    BCC NoCol
    JSR InitGame
NoCol    
    STA CXCLR

; Frame Stuff
    INC frameCounter
    LDA frameCounter
    AND #$01
    BNE NoJoy
    JSR joy
NoJoy

    LDA SWCHB
    CMP switchesRead
    BEQ SwitchesDone
    STA switchesRead
    LSR
    BCS NoReset         ; N: Continue
GameOver
    LDA #$00
    STA level
    BEQ GroundZero
NoReset             
    LSR
    BCS SwitchesDone
    INC level
    LDA level
    EOR #MAXLEVELS
    BNE GroundZero
    STA level

GroundZero
    JSR InitGame

SwitchesDone

    LDA #$01
    JSR AddScore

    LDA frameCounter
    LSR
    LSR
    LSR
    LSR
    AND #$0F
    CMP shiftval
    JMP NoChange
;    BEQ NoChange
    STA shiftval
    TAX
    LDA shiftaddtab,X
    CLC
    ADC verPosBalloon
    STA verPosBalloon
NoChange
  
; Limit movement
    LDA verPosBalloon
    CMP #VERBOTTOMLIMIT
    BCS PosFine1
    LDA #VERBOTTOMLIMIT
PosFine1
    CMP #VERTOPLIMIT+1
    BCC PosFine2
    LDA #VERTOPLIMIT
PosFine2
    STA verPosBalloon

    LDA #HORRIGHTLIMIT
    CMP horPosBalloon
    BCS PosFine3
    STA horPosBalloon
PosFine3

; Enemy Animation
    INC enemyAnim

; Balloon Animation
    LDA frameCounter
    AND #$07
    BNE BalloonAnimDone

    INC lives
    LDA lives
    EOR #MAXLIVES
    BNE LivesDone
    STA lives
LivesDone

    INC levelTimer
    LDA levelTimer
    EOR #MAXTIMER
    BNE LTDone
    STA levelTimer
LTDone

    INC balloonAnim
    LDA balloonAnim
    EOR #BALLOONFRAMES
    BNE BalloonAnimDone
    STA balloonAnim
BalloonAnimDone

.waitoverscan
    LDA INTIM
    BNE .waitoverscan
    JMP NEWSCREEN

joy     
        LDA SWCHA      
        ASL
        BCS RightDone
        INC horPosBalloon
RightDone
        ASL
        BCS LeftDone
        DEC horPosBalloon
LeftDone
        ASL
        BCS DownDone
        DEC verPosBalloon
DownDone
        ASL
        BCS UpDone
        INC verPosBalloon
UpDone
        RTS

AddScore
    SED
    CLC
    ADC bcdScore+2
    STA bcdScore+2
    LDA bcdScore+1
    ADC #$00
    STA bcdScore+1
    LDA bcdScore
    ADC #$00
    STA bcdScore
    CLD
    RTS

    ORG $F88B

FACE_0
                                .byte   %00000011
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
FACE_1
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111

FACE_2
                                .byte   %00000011 ; shared
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00010011
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111

FACE_3
                                .byte   %00000011 ; shared
                                .byte   %00000111
                                .byte   %00100111
                                .byte   %00010011
                                .byte   %00100111
                                .byte   %00000111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
FACE_4
                                .byte   %00000011
                                .byte   %01000111
                                .byte   %00100111
                                .byte   %00000011
                                .byte   %00100111
                                .byte   %01000111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
FACE_5
                                .byte   %10000011
                                .byte   %01000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000111
                                .byte   %01000111
                                .byte   %10001111
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
FACE_6
                                .byte   %00000011
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000001
                                .byte   %00000101
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000011

    ORG $F900

divideby3
    .byte 0,0,0,0
    .byte 1,1,1
    .byte 2,2,2
    .byte 3,3,3
    .byte 4,4,4
    .byte 5,5,5
    .byte 6,6,6
    .byte 7,7,7
    .byte 8,8,8
    .byte 9,9,9
    .byte 10,10,10
    .byte 11,11,11
    .byte 12,12,12
    .byte 13,13,13
    .byte 14,14,14
    .byte 15,15,15
    .byte 16,16,16
    .byte 17,17,17
    .byte 18,18,18
    .byte 19,19,19
    .byte 20,20,20
    .byte 21,21,21
    .byte 22,22,22
    .byte 23,23,23
    .byte 24,24,24
    .byte 25,25,25

    ORG $FA00
BALLOONLEVEL0_0
                                .byte   %00000000
                                .byte   %00110000
                                .byte   %01111010
                                .byte   %01101010
                                .byte   %01101011
                                .byte   %01111011
                                .byte   %01111011
                                .byte   %01110011
                                .byte   %01011010
                                .byte   %01011010
                                .byte   %01111011
                                .byte   %01110011
                                .byte   %00000001
                                .byte   %00000000
                                .byte   %00000110
                                .byte   %00000111
                                .byte   %00001111
                                .byte   %00001001
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001001
                                .byte   %00001111
                                .byte   %00001111
                                .byte   %00000111
                                .byte   %00000011
                                .byte   %00000000

BALLOONLEVEL0_1
                                .byte   %00000000
                                .byte   %11100000
                                .byte   %11111101
                                .byte   %00111101
                                .byte   %00100101
                                .byte   %00100101
                                .byte   %01100101
                                .byte   %01101101
                                .byte   %01101101
                                .byte   %01101101
                                .byte   %01101101
                                .byte   %00001101
                                .byte   %00000000
                                .byte   %00011000
                                .byte   %01011010
                                .byte   %01011010
                                .byte   %11011010
                                .byte   %11001110
                                .byte   %11011110
                                .byte   %11011010
                                .byte   %01011010
                                .byte   %01011010
                                .byte   %11011110
                                .byte   %11011110
                                .byte   %10001100
                                .byte   %00000000

BALLOONLEVEL0_2
                                .byte   %00000000
                                .byte   %10000000
                                .byte   %10000001
                                .byte   %00110011
                                .byte   %01110011
                                .byte   %01111111
                                .byte   %11111110
                                .byte   %11011110
                                .byte   %11011110
                                .byte   %11011110
                                .byte   %11010011
                                .byte   %01110011
                                .byte   %01100000
                                .byte   %00000000
                                .byte   %10111000
                                .byte   %10111110
                                .byte   %10111110
                                .byte   %10011000
                                .byte   %10011100
                                .byte   %10001101
                                .byte   %10001101
                                .byte   %10111101
                                .byte   %10111101
                                .byte   %10011101
                                .byte   %00000001
                                .byte   %00000000

BALLOONLEVEL0_3
                                .byte   %00000000
                                .byte   %01000000
                                .byte   %01001001
                                .byte   %01101001
                                .byte   %01101011
                                .byte   %01111011
                                .byte   %01111011
                                .byte   %01111011
                                .byte   %01011011
                                .byte   %01011001
                                .byte   %01001001
                                .byte   %00001000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000011
                                .byte   %00000011
                                .byte   %00000011
                                .byte   %00000011
                                .byte   %00000011
                                .byte   %00000111
                                .byte   %00000111
                                .byte   %00000101
                                .byte   %00000100
                                .byte   %00000100
                                .byte   %00000000

BALLOONLEVEL1_0
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111110
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111100
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000

BALLOONLEVEL1_1
                                .byte   %11111111
                                .byte   %00000111
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00001100
                                .byte   %11111100
                                .byte   %11100000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %10000001
                                .byte   %11000011
                                .byte   %01111100
                                .byte   %00111000
                                .byte   %00010000
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000011
                                .byte   %00000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %11100000
                                .byte   %11111000

BALLOONLEVEL1_2
                                .byte   %10000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000111
                                .byte   %00011111
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %00011000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000010
                                .byte   %00111100
                                .byte   %00111100
                                .byte   %00001100
                                .byte   %00000100
                                .byte   %00000010
                                .byte   %00000000
                                .byte   %10000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %11111100
                                .byte   %11111111

BALLOONLEVEL1_3
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %11111100
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111110
                                .byte   %11111111

BALLOONLEVEL2_0
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11111111
                                .byte   %11111111

    ORG $FB00

BALLOONLEVEL2_1
                                .byte   %11000000
                                .byte   %10000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000111
                                .byte   %00011111
                                .byte   %00011111
                                .byte   %00001110
                                .byte   %00000110
                                .byte   %00000110
                                .byte   %00000110
                                .byte   %00000110
                                .byte   %10001110
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %00111110
                                .byte   %00001110
                                .byte   %00000110
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %11000000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11110000
                                .byte   %11111111

BALLOONLEVEL2_2
                                .byte   %11111111 
                                .byte   %10000001
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00100000
                                .byte   %01111000
                                .byte   %11101111
                                .byte   %01000111
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %11111000
                                .byte   %00011100
                                .byte   %00001110
                                .byte   %00000110
                                .byte   %00000001
                                .byte   %00000000
                                .byte   %11000000
                                .byte   %11100000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111111

BALLOONLEVEL2_3
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111100
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111100
                                .byte   %11111111
                                .byte   %11111111

BALLOONLEVEL3_0
                                .byte   %11111111
                                .byte   %11111100
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11111100
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111111

BALLOONLEVEL3_1
                                .byte   %11111111
                                .byte   %00111100
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %11000010
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %11000000
                                .byte   %10000000
                                .byte   %00000000
                                .byte   %00100000
                                .byte   %01100000
                                .byte   %11100000
                                .byte   %11111000
                                .byte   %11111110
                                .byte   %00001110
                                .byte   %00000010
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %01100000
                                .byte   %11100000
                                .byte   %11110000
                                .byte   %11111111

BALLOONLEVEL3_2
                                .byte   %11111111
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00100000
                                .byte   %11111110
                                .byte   %11100000
                                .byte   %10000000
                                .byte   %10000000
                                .byte   %10000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %10000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %10000000
                                .byte   %11111111 
                                .byte   %01111111
                                .byte   %00011111 
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %10000000
                                .byte   %11111111

BALLOONLEVEL3_3
                                .byte   %11111111
                                .byte   %11111100
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111100
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11111111


LEVELDATATAB
    .word BALLOONLEVEL0_0, BALLOONLEVEL0_1, BALLOONLEVEL0_2, BALLOONLEVEL0_3
    .byte $68
    .byte $8A
    .byte $01
    .word BALLOONLEVEL1_0, BALLOONLEVEL1_1, BALLOONLEVEL1_2, BALLOONLEVEL1_3
    .byte $18
    .byte $80
    .byte $00
    .word BALLOONLEVEL2_0, BALLOONLEVEL2_1, BALLOONLEVEL2_2, BALLOONLEVEL2_3
    .byte $18
    .byte $2B
    .byte $00
    .word BALLOONLEVEL3_0, BALLOONLEVEL3_1, BALLOONLEVEL3_2, BALLOONLEVEL3_3
    .byte $90
    .byte $80
    .byte $00

ENEMYDATATAB
; No Enemy
    .byte $00, $00, $00
    .word $0000

; Face
    .byte $4C, $1D, $10
    .word FACE_0

; Spike
;    .byte $20, $20, 34
;    .word FACE_0

    ORG $FC00

DrawScore
; slightly modified 48 pixel version:
loop2
        LDY lines           ;+3  63  189
        SLEEP 4
        LDA     (charPtr),y     ;+5  68  204
        STA     WSYNC           ;go
        STA.w   GRP0            ;+3  71  213      D1     --      --     --
        LDA     (charPtr+$2),y  ;+5   5   15
        STA     GRP1            ;+3   8   24      D1     D1      D2     --
        LDA     (charPtr+$4),y  ;+5  13   39
        STA     GRP0            ;+3  16   48      D3     D1      D2     D2
        LDA     (charPtr+$6),y  ;+5  21   63
        STA     tempVar1         ;+3  24   72
        LAX     (charPtr+$8),y  ;+5  29   87
        LDA     (charPtr+$A),y  ;+5  36  108
        LDY     tempVar1         ;+3  41  123              !
        STY     GRP1            ;+3  44  132      D3     D3      D4     D2!
        STX     GRP0            ;+3  47  141      D5     D3!     D4     D4
        STA     GRP1            ;+3  50  150      D5     D5      D6     D4!
        STA     GRP0            ;+3  53  159      D4*    D5!     D6     D6
        DEC     lines           ;+5  58  174                             !
        BPL     loop2           ;+2  60  180
        LDY #0
        STY GRP0
        STY GRP1
        STY GRP0
        STY NUSIZ0
        STY NUSIZ1
        STY VDELP0
        STY VDELP1

        LDA #$32           ;
        STA COLUP0         ; Sprite colour
        LDA #$1E           ;
        STA COLUP1         ; Sprite colour
        STA REFP1

        LDA horPosBalloon
        SEC
        STA WSYNC                ;
.waitzonex
        SBC #$0F                 ;
        BCS .waitzonex           ; RESP loop
        EOR #$07                 ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        STA HMP0
        STA RESP0
        LDA horPosEnemy
PosElement
        SEC
        STA WSYNC                ;
.waitzone
        SBC #$0F                 ;
        BCS .waitzone            ; RESP loop
        EOR #$07                 ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        STA HMP1
        STA RESP1
        STA WSYNC
        STA HMOVE
        RTS                      ; done, that's all!

InitGame
    LDX level
    LDY LEVELTYPE,X
    LDX #DATASIZE-1
CopyLevelData
    LDA LEVELDATATAB,Y
    STA PFPtr,X
    DEY
    DEX
    BPL CopyLevelData

    LDX enemyType
InitEnemy
    LDY ENEMYTYPE,X
    LDX #ENEMYDATASIZE-1
CopyEnemyData
    LDA ENEMYDATATAB,Y
    STA verPosEnemy,X
    DEY
    DEX
    BPL CopyEnemyData
    LDA $7777
    LDA enemyType
    LDA enemyHeight    
    RTS

zero
    .byte %00111000
    .byte %01000100
    .byte %01000100
    .byte %01000100
    .byte %01000100
    .byte %01000100
    .byte %00111000

one
    .byte %00111000
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00110000
    .byte %00010000
    
two
    .byte %01111100
    .byte %01000000
    .byte %00100000
    .byte %00011000
    .byte %00000100
    .byte %01000100
    .byte %00111000

three
    .byte %00111000
    .byte %01000100
    .byte %00000100
    .byte %00011000
    .byte %00001000
    .byte %00000100
    .byte %01111100

four
    .byte %00001000
    .byte %00001000
    .byte %01111100
    .byte %01001000
    .byte %00101000
    .byte %00011000
    .byte %00001000

five
    .byte %00111000
    .byte %01000100
    .byte %00000100
    .byte %00000100
    .byte %01111000
    .byte %01000000
    .byte %01111100

six
    .byte %00111000
    .byte %01000100
    .byte %01000100
    .byte %01111000
    .byte %01000000
    .byte %00100000
    .byte %00011100

seven
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00001000
    .byte %00000100
    .byte %01111100

eight
    .byte %00111000
    .byte %01000100
    .byte %01000100
    .byte %00111000
    .byte %01000100
    .byte %01000100
    .byte %00111000

nine
    .byte %01110000
    .byte %00001000
    .byte %00000100
    .byte %00111100
    .byte %01000100
    .byte %01000100
    .byte %00111000

balloonAnimTab
    .byte <BALLOON14 - KERNEL_HEIGHT - 1, <BALLOON14 - KERNEL_HEIGHT - 1
    .byte <BALLOON16 - KERNEL_HEIGHT - 1, <BALLOON17 - KERNEL_HEIGHT - 1
    .byte <BALLOON18 - KERNEL_HEIGHT - 1, <BALLOON17 - KERNEL_HEIGHT - 1
    .byte <BALLOON16 - KERNEL_HEIGHT - 1, <BALLOON14 - KERNEL_HEIGHT - 1
    .byte <BALLOON14 - KERNEL_HEIGHT - 1, <BALLOON16 - KERNEL_HEIGHT - 1
    .byte <BALLOON17 - KERNEL_HEIGHT - 1, <BALLOON18 - KERNEL_HEIGHT - 1
    .byte <BALLOON17 - KERNEL_HEIGHT - 1, <BALLOON16 - KERNEL_HEIGHT - 1

    ORG $FD90

    .byte $00
BALLOONLLL
    .byte $00
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $F0
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $50
    .byte $00


    .byte $00
BALLOONLL
    .byte $00
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $40
    .byte $00

    .byte $00
BALLOONL
    .byte $00
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $30
    .byte $00

balloonMoveTabHigh
    .byte >BALLOONLLL,>BALLOONLLL
    .byte >BALLOONLL,>BALLOONL
    .byte >BALLOONM,>BALLOONR
    .byte >BALLOONRR,>BALLOONRRR
    .byte >BALLOONRRR,>BALLOONRR
    .byte >BALLOONR,>BALLOONM
    .byte >BALLOONL,>BALLOONLL

balloonMoveTab
    .byte <BALLOONLLL - KERNEL_HEIGHT - 1, <BALLOONLLL - KERNEL_HEIGHT - 1
    .byte <BALLOONLL - KERNEL_HEIGHT - 1, <BALLOONL - KERNEL_HEIGHT - 1
    .byte <BALLOONM - KERNEL_HEIGHT - 1, <BALLOONR - KERNEL_HEIGHT - 1
    .byte <BALLOONRR - KERNEL_HEIGHT - 1, <BALLOONRRR - KERNEL_HEIGHT - 1
    .byte <BALLOONRRR - KERNEL_HEIGHT - 1, <BALLOONRR - KERNEL_HEIGHT - 1
    .byte <BALLOONR - KERNEL_HEIGHT - 1, <BALLOONM - KERNEL_HEIGHT - 1
    .byte <BALLOONL - KERNEL_HEIGHT - 1, <BALLOONLL - KERNEL_HEIGHT - 1

    ORG $FE90

    .byte $00
BALLOONM
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
BALLOONR
    .byte $00
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $D0
    .byte $00


    .byte $00
BALLOONRR
    .byte $00
    .byte $10
    .byte $10
    .byte $10
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $C0
    .byte $00

    .byte $00
BALLOONRRR
    .byte $00
    .byte $10
    .byte $10
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

    .byte $00
    .byte $00
    .byte $00
    .byte $10
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $B0
    .byte $00


shifttab
        .byte 0,1,2,3,4,5,6,7,8,7,6,5,4,3,2,1

shiftaddtab
        .byte -1,1,1,1,1,1,1,1,1,-1,-1,-1,-1,-1,-1,-1

digittab
    .byte #<zero,#<one,#<two,#<three,#<four
    .byte #<five,#<six,#<seven,#<eight,#<nine

TIMERPF1
    .byte $00,$80,$C0,$E0,$F0,$F8,$FC,$FE,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
TIMERPF2
    .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$03,$07,$0F,$1F,$3F,$7F,$FF,$FF

LIVESOFFSET
    .byte $0,$76,$66,$56

TIMERCOLOR
    .byte $80,$80,$88,$8A,$8C,$8F
    
TIMERSHAPE
    .byte $00,$80,$C0,$E0

LEVELTYPE
    .byte 0*DATASIZE+DATASIZE-1
    .byte 1*DATASIZE+DATASIZE-1
    .byte 2*DATASIZE+DATASIZE-1
    .byte 3*DATASIZE+DATASIZE-1

ENEMYTYPE
    .byte 0*ENEMYDATASIZE+ENEMYDATASIZE-1
    .byte 1*ENEMYDATASIZE+ENEMYDATASIZE-1

;LEVELTYPE
;    .byte 0*DATASIZE+DATASIZE-1
;    .byte 1*DATASIZE+DATASIZE-1
;    .byte 1*DATASIZE+DATASIZE-1
;    .byte 2*DATASIZE+DATASIZE-1
;    .byte 1*DATASIZE+DATASIZE-1
;    .byte 2*DATASIZE+DATASIZE-1
;    .byte 3*DATASIZE+DATASIZE-1
;    .byte 2*DATASIZE+DATASIZE-1
;    .byte 3*DATASIZE+DATASIZE-1
;    .byte 1*DATASIZE+DATASIZE-1
;    .byte 1*DATASIZE+DATASIZE-1
;    .byte 2*DATASIZE+DATASIZE-1
;    .byte 1*DATASIZE+DATASIZE-1
;    .byte 2*DATASIZE+DATASIZE-1
;    .byte 3*DATASIZE+DATASIZE-1
;    .byte 2*DATASIZE+DATASIZE-1
;    .byte 3*DATASIZE+DATASIZE-1

faceAnimTab
    .byte <FACE_0 - KERNEL_HEIGHT - 1
    .byte <FACE_0 - KERNEL_HEIGHT - 1
    .byte <FACE_0 - KERNEL_HEIGHT - 1
    .byte <FACE_0 - KERNEL_HEIGHT - 1
    .byte <FACE_1 - KERNEL_HEIGHT - 1
    .byte <FACE_2 - KERNEL_HEIGHT - 1
    .byte <FACE_3 - KERNEL_HEIGHT - 1
    .byte <FACE_4 - KERNEL_HEIGHT - 1
    .byte <FACE_5 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1
    .byte <FACE_6 - KERNEL_HEIGHT - 1

    ORG $FF80

        .byte %00000000
BALLOON18
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %00010000

        .byte %00000000
        .byte %00000000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00000000


        .byte %00000000
BALLOON17
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00000000

        .byte %00000000
        .byte %00000000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %00010000
        .byte %00000000


        .byte %00000000
BALLOON16
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %00010000
        .byte %00000000

        .byte %00000000
        .byte %00000000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00000000
        .byte %00000000


        .byte %00000000
BALLOON14
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %00010000
        .byte %00000000
        .byte %00000000

        .byte %00000000
        .byte %00000000 
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00000000
        .byte %00000000
        .byte %00000000

LIVESSHAPE
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00010000

LIVESNUSIZ
    .byte $00,$00,$01,$03

    ORG $FFFA
    .WORD Start
    .WORD Start
    .WORD Start